import {IControllerClass} from "./definitions";
import {applyConfiguration, addRouteMiddleware} from "./internal";
import ExpressAdapter from "./ExpressAdapter";

export class Bootstrapper {
    private controllers: IControllerClass[] = [];
    private adapter: ExpressAdapter;

    constructor(private app: any, services: string[]) {
        this.adapter = new ExpressAdapter(app, services);
    }

    /**
     * Adds a new decorator with given name and vararg arguments.
     *
     * @param  {string}       name CamelCase name.
     * @return {Bootstrapper}      [description]
     */
    public static middleware(generateMiddleware: (values: any[]) => any): any {
        return function(...values: any[]): any {
            return function(targetClass: any, methodName: string,
                descriptor: TypedPropertyDescriptor<any>): TypedPropertyDescriptor<any> {
                let middleware: any = generateMiddleware(values);

                addRouteMiddleware(targetClass, methodName, middleware);

                return descriptor;
            };
        };
    }

    /**
     * Install an API controller.
     * @param  {any} controllerClass the class
     */
    public controller(controllerClass: any): Bootstrapper {
        this.controllers.push(controllerClass);

        applyConfiguration(this.adapter, controllerClass);

        return this;
    }
}
