"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
function Log() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i - 0] = arguments[_i];
    }
    switch (args.length) {
        case 1:
            return logClass.apply(this, args);
        case 2:
            return logProperty.apply(this, args);
        case 3:
            if (typeof args[2] === "number") {
                return logParameter.apply(this, args);
            }
            return logMethod.apply(this, args);
        default:
            throw new Error("Decorators are not valid here.");
    }
}
exports.Log = Log;
function logMethod(targetClass, methodName, descriptor) {
    var originalMethod = descriptor.value;
    descriptor.value = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        var metadataKey = "log_" + methodName + "_parameters";
        var indices = targetClass[metadataKey];
        if (Array.isArray(indices)) {
            for (var i = 0; i < args.length; ++i) {
                if (indices.indexOf(i) !== -1) {
                    var arg = args[i];
                    var argStr = JSON.stringify(arg) || arg.toString();
                    console.log(methodName + " arg[" + i + "]: " + argStr);
                }
            }
            var result = originalMethod.apply(this, args);
            return result;
        }
        else {
            var a = args.map(function (arg) {
                try {
                    return JSON.stringify(arg);
                }
                catch (err) {
                    return "!!ERROR: " + err.message + "!!";
                }
            }).join();
            var result = originalMethod.apply(this, args);
            var r = JSON.stringify(result);
            console.log("Call: " + methodName + "(" + a + ") => " + r);
            return result;
        }
    };
    return descriptor;
}
function logParameter(targetClass, methodName, index) {
    var metadataKey = "log_" + methodName + "_parameters";
    if (Array.isArray(targetClass[metadataKey])) {
        targetClass[metadataKey].push(index);
    }
    else {
        targetClass[metadataKey] = [index];
    }
}
function logClass(targetClass) {
    var originalClass = targetClass;
    return (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            console.log("new: " + originalClass.name);
            _super.apply(this, args);
        }
        return class_1;
    }(targetClass));
}
function logProperty(targetClass, propertyName) {
    var value = this[propertyName];
    var getter = function () {
        console.log("Get: " + propertyName + " => " + value);
        return value;
    };
    var setter = function (newValue) {
        console.log("Set: " + propertyName + " <= " + newValue + " (was " + value + ")");
        value = newValue;
    };
    if (delete this[propertyName]) {
        Object.defineProperty(targetClass, propertyName, {
            configurable: true,
            enumerable: true,
            get: getter,
            set: setter,
        });
    }
}
//# sourceMappingURL=debug.js.map