import * as express from "express";

/**
 * Request parameter types.
 */
export enum ParameterType {
    PATH_PARAMETER,
    BODY_PARAMETER,
    MULTIPART_PARAMETER,
    QUERY_PARAMETER,
    HEADER_PARAMETER,
    SERVICE_PARAMETER,
    COOKIE_PARAMETER
}

/**
 * Configuration of a decorated parameter: index of the
 * parameter
 */
export interface IParameterConfiguration {
    index: number;
    name?: string;
    type: ParameterType;
    valueType?: Function;
    required?: boolean;
}

/**
 * A request header.
 */
export interface IHeader {
    field: string;
    value: string;
}

/**
 * Data output types.
 */
export enum SendType {
    JSON
}

export interface IControllerClass extends Function {
    prototype: IObjectWithControllerConfiguration;
    new (): Function;
}

/**
 * Configuration of a route handler.
 */
export interface IRoute {
    method: string;
    path: string;
    handlerName: string;
}

/**
 * Configiration of a middleware handler.
 */
export interface IMiddleware {
    path?: string;
    handlerName: string;
}

/**
 * Configuration of an API controller.
 *
 * Contains arrays of routes and middlewares, root path,
 * parameter configuration per method and send types per method.
 */
export interface IControllerConfiguration {
    routes: IRoute[];
    middlewares: IMiddleware[];
    root: string;
    methodsParameters: { [methodName: string]: IParameterConfiguration[] };
    sendTypes: { [methodName: string]: SendType };
    routesMiddlewares: { [methodName: string]: express.RequestHandler[] };
}

/**
 * An interface for any class that uses the controller decorator.
 * A class will have the properties in this interface.
 */
export interface IObjectWithControllerConfiguration {
    $$controllerConfiguration: IControllerConfiguration;
}

/**
 * Data sent to the adapter.
 *
 * TODO: abstractify?
 */
export interface IAdapterData {
    req: express.Request;
    res: express.Response;
}
