"use strict";
var Promise = require("bluebird");
var domain = require("domain");
var definitions_1 = require("./definitions");
function addClassConfiguration(target) {
    if (!target.$$controllerConfiguration) {
        target.$$controllerConfiguration = {
            methodsParameters: {},
            middlewares: [],
            root: null,
            routes: [],
            routesMiddlewares: {},
            sendTypes: {},
        };
    }
}
function addMethodDecorator(target, methodName, parameterConfiguration) {
    addClassConfiguration(target);
    if (!target.$$controllerConfiguration.methodsParameters[methodName]) {
        target.$$controllerConfiguration.methodsParameters[methodName] = [];
    }
    target.$$controllerConfiguration.methodsParameters[methodName].push(parameterConfiguration);
}
function addRouteMiddleware(target, methodName, middleware) {
    addClassConfiguration(target);
    if (!target.$$controllerConfiguration.routesMiddlewares[methodName]) {
        target.$$controllerConfiguration.routesMiddlewares[methodName] = [];
    }
    target.$$controllerConfiguration.routesMiddlewares[methodName].push(middleware);
}
exports.addRouteMiddleware = addRouteMiddleware;
function ControllerDec(root) {
    return function (target) {
        addClassConfiguration(target.prototype);
        target.prototype.$$controllerConfiguration.root = root;
        return target;
    };
}
exports.ControllerDec = ControllerDec;
function SendJsonDec(target, methodName, descriptor) {
    addClassConfiguration(target);
    target.$$controllerConfiguration.sendTypes[methodName] = definitions_1.SendType.JSON;
}
exports.SendJsonDec = SendJsonDec;
function parameterDecoratorFactory(paramType) {
    return function () {
        return function (_target, methodName, index) {
            var target = _target;
            var paramConfig = { index: index, type: paramType };
            addMethodDecorator(target, methodName, paramConfig);
        };
    };
}
exports.parameterDecoratorFactory = parameterDecoratorFactory;
function parameterDecoratorWithNameFactory(paramType) {
    return function (name, type, required) {
        return function (_target, methodName, index) {
            var target = _target;
            var paramConfig = {
                index: index,
                name: name,
                type: paramType,
                valueType: type,
                required: required,
            };
            addMethodDecorator(target, methodName, paramConfig);
        };
    };
}
exports.parameterDecoratorWithNameFactory = parameterDecoratorWithNameFactory;
function methodDecoratorFactory(method) {
    return function (path) {
        return function (target, handlerName, descriptor) {
            addClassConfiguration(target);
            var route = { method: method, path: path, handlerName: handlerName };
            target.$$controllerConfiguration.routes.push(route);
        };
    };
}
exports.methodDecoratorFactory = methodDecoratorFactory;
function callRequestHandler(adapter, handler, controller, configuration, handlerName, adapterRequestData, next) {
    var paramList = [];
    if (configuration.methodsParameters[handlerName]) {
        configuration.methodsParameters[handlerName].forEach(function (paramConfig) {
            paramList[paramConfig.index] = adapter.getParameterWithConfig(paramConfig, adapterRequestData);
        });
    }
    paramList.push(next);
    var dom = domain.create();
    dom.on("error", next);
    dom.run(function () {
        var result = handler.apply(controller, paramList);
        if (result != null) {
            var wrappedResult = result;
            if (!(wrappedResult instanceof ResponseMetadata)) {
                wrappedResult = new ResponseMetadata(200, wrappedResult);
            }
            callSendMethod(adapter, handler, wrappedResult, configuration.sendTypes[handlerName], adapterRequestData, next);
        }
    });
}
exports.callRequestHandler = callRequestHandler;
function unwrapAsyncValue(promise, cb) {
    Promise.resolve(promise)
        .done(function (unwrappedValue) { return typeof unwrappedValue === "function" ? unwrappedValue(cb) : cb(null, unwrappedValue); }, cb);
}
function callSendMethod(adapter, handler, response, sendType, adapterRequestData, next) {
    unwrapAsyncValue(response.body, function (err, body) {
        if (err) {
            return next(err);
        }
        switch (sendType) {
            case definitions_1.SendType.JSON:
                adapter.sendJson(response.statusCode, body, adapterRequestData, response.headers);
                break;
            default:
                adapter.send(response.statusCode, body, adapterRequestData, response.headers);
                break;
        }
    });
}
function createPathWithRoot(root, path) {
    if (root && path) {
        if (root[root.length - 1] !== "/" && path[0] !== "/") {
            return root + "/" + path;
        }
        else if (root[root.length - 1] === "/" && path[0] === "/") {
            return root + path.slice(1);
        }
        else {
            return root + path;
        }
    }
    else if (root) {
        return root;
    }
    else {
        return path;
    }
}
function applyConfiguration(adapter, controllerClass) {
    var config = controllerClass.prototype.$$controllerConfiguration;
    if (config === undefined) {
        throw new Error("Supplied controller is not a decorated controller.");
    }
    var instance = new controllerClass();
    config.middlewares.forEach(function (middleware) { return adapter.addMiddleware(config, createPathWithRoot(config.root, middleware.path), instance, middleware.handlerName, instance[middleware.handlerName]); });
    config.routes.forEach(function (route) { return adapter.addRoute(config, route.method, createPathWithRoot(config.root, route.path), instance, route.handlerName, instance[route.handlerName], config.routesMiddlewares[route.handlerName]); });
}
exports.applyConfiguration = applyConfiguration;
var ResponseMetadata = (function () {
    function ResponseMetadata(statusCodeOrBody, maybeBody) {
        if (typeof statusCodeOrBody === "number") {
            _a = [statusCodeOrBody, maybeBody], this.statusCode = _a[0], this.body = _a[1];
        }
        else {
            _b = [200, statusCodeOrBody], this.statusCode = _b[0], this.body = _b[1];
        }
        var _a, _b;
    }
    return ResponseMetadata;
}());
//# sourceMappingURL=internal.js.map