import {ParameterType} from "./definitions";
import {parameterDecoratorFactory, parameterDecoratorWithNameFactory} from "./internal";
import {methodDecoratorFactory, ControllerDec, SendJsonDec} from "./internal";

/*import Authentication from "../lib/Authentication";*/

/**
 * Create an API controller.
 *
 * @param  {string} root Root of the router.
 */
export const Controller: any = ControllerDec;

/**
 * Forces the response to be in JSON format.
 *
 * Usage: @SendJson
 */
export const SendJson: any = SendJsonDec;

/**
 * Retrieve an object of all services.
 *
 * Usage: @Services
 */
export const Services: any = parameterDecoratorFactory(ParameterType.SERVICE_PARAMETER);

/**
 * When supplied with a name, the given parameter. Otherwise, an object of
 * all body contents.
 *
 * @param {string} Body key value
 * @param {type} [type] Optional enforced type
 * @param {required} [required=false] Optional required
 */
export const Body: any = parameterDecoratorWithNameFactory(ParameterType.BODY_PARAMETER);

/**
 * A multipart form parser.
 *
 * Usage: @Multipart()
 *
 * Returns a Promise with { fields:, files: }
 */
export const Multipart: any = parameterDecoratorWithNameFactory(ParameterType.MULTIPART_PARAMETER);

/**
 * A router path parameter.
 * @param {string} Path parameter name
 * @param {type} [type] Optional enforced type
 * @param {required} [required=false] Optional required
 */
export const Path: any = parameterDecoratorWithNameFactory(ParameterType.PATH_PARAMETER);

/**
 * A query parameter.
 *
 * For `?foo=bar`, use @Query("foo")
 *
 * @param {string} Queryname
 * @param {type} [type] Optional enforced type
 * @param {required} [required=false] Optional required
 */
export const Query: any = parameterDecoratorWithNameFactory(ParameterType.QUERY_PARAMETER);

/**
 * A header value.
 *
 * The name is case insensitive.
 *
 * @param {string} Name of headerfield
 * @param {type} [type] Optional enforced type
 * @param {required} [required=false] Optional required
 */
export const Header: any = parameterDecoratorWithNameFactory(ParameterType.HEADER_PARAMETER);

/**
 * A cookie value.
 * @param {string} Cookiename
 * @param {type} [type] Optional enforced type
 * @param {required} [required=false] Optional required
 */
export const Cookie: any = parameterDecoratorWithNameFactory(ParameterType.COOKIE_PARAMETER);

/**
 * A service. Possible services are set up in the bootstrapper.
 *
 * @param {string} Servicename
 */
export const Service: any = parameterDecoratorWithNameFactory(ParameterType.SERVICE_PARAMETER);

/**
 * Make this method a GET method handler.
 *
 * @param {string} path Path of the route.
 */
export const Get: any = methodDecoratorFactory("get");

/**
 * Make this method a GET method handler.
 *
 * @param {string} path Path of the route.
 */
export const Post: any = methodDecoratorFactory("post");

/**
 * Make this method a PUT method handler.
 *
 * @param {string} path Path of the route.
 */
export const Put: any = methodDecoratorFactory("put");

/**
 * Make this method a DELETE method handler.
 *
 * @param {string} path Path of the route.
 */
export const Delete: any = methodDecoratorFactory("delete");

/**
 * Make this method a PATCH method handler.
 *
 * @param {string} path Path of the route.
 */
export const Patch: any = methodDecoratorFactory("patch");
